// FILE: ruinREG.c

/* estimating the probability of ruin for variable premiums by regenerative
   simulation */

// Mik Cieslak & Kris Luttmer

#include "qmc.h"
#include "ruinREG.h"
#include "Dist.h"
#include <math.h>

// -------------------------------------------------------------------------

void RuinREG (double lambda, double mu, double c, double delta, double u,
	      double *D, double *sumTi)
{
  int i;
  int done;
  double T, Y, r, interm;
  double X[2];

  *D = 0.0;
  *sumTi = 0.0; 
  
  X[0] = u;
  done = 0;
  i = 0;  

  while (!done)
  {
    T = Expon (1.0 / lambda, RPoint(i));
    Y = Expon (mu, RPoint (++i));

    *sumTi = *sumTi + T;

    interm = X[0] * exp(-delta*T)-c*((1.0-exp(-delta*T))/delta)+Y;

    if (Y > 0.0)
    {
      if (Y > interm)
        X[1] = Y;
      else
        X[1] = interm;
    }
    else if (interm > 0.0)
      X[1] = interm;
    else
      X[1] = 0.0;
    
    r = (log((X[0]+c/delta)/(u+c/delta)))/delta;

    if (X[0] > u)
    {
      if (T-r > 0.0)
      {
	*sumTi = *sumTi - (T-r);
        done = 1;
      }
    }
    else
      *D = *D + T;

    X[0] = X[1];
    i++;
  }
  
  *D = *sumTi - *D;

  return;
} 

// -------------------------------------------------------------------------
